@@ -18,13 +18,13 @@
 %                    Based on kmiya's sixel (2014-03-28)                      %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2015 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2020 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -40,39 +40,39 @@
 /*
   Include declarations.
 */
-#include "magick/studio.h"
-#include "magick/attribute.h"
-#include "magick/blob.h"
-#include "magick/blob-private.h"
-#include "magick/cache.h"
-#include "magick/color.h"
-#include "magick/color-private.h"
-#include "magick/colormap.h"
-#include "magick/colorspace.h"
-#include "magick/colorspace-private.h"
-#include "magick/exception.h"
-#include "magick/exception-private.h"
-#include "magick/geometry.h"
-#include "magick/image.h"
-#include "magick/image-private.h"
-#include "magick/list.h"
-#include "magick/magick.h"
-#include "magick/memory_.h"
-#include "magick/monitor.h"
-#include "magick/monitor-private.h"
-#include "magick/pixel-accessor.h"
-#include "magick/pixel-private.h"
-#include "magick/quantize.h"
-#include "magick/quantum-private.h"
-#include "magick/resize.h"
-#include "magick/resource_.h"
-#include "magick/splay-tree.h"
-#include "magick/static.h"
-#include "magick/string_.h"
-#include "magick/thread-private.h"
-#include "magick/module.h"
-#include "magick/threshold.h"
-#include "magick/utility.h"
+#include "MagickCore/studio.h"
+#include "MagickCore/attribute.h"
+#include "MagickCore/blob.h"
+#include "MagickCore/blob-private.h"
+#include "MagickCore/cache.h"
+#include "MagickCore/color.h"
+#include "MagickCore/color-private.h"
+#include "MagickCore/colormap.h"
+#include "MagickCore/colorspace.h"
+#include "MagickCore/colorspace-private.h"
+#include "MagickCore/exception.h"
+#include "MagickCore/exception-private.h"
+#include "MagickCore/geometry.h"
+#include "MagickCore/image.h"
+#include "MagickCore/image-private.h"
+#include "MagickCore/list.h"
+#include "MagickCore/magick.h"
+#include "MagickCore/memory_.h"
+#include "MagickCore/monitor.h"
+#include "MagickCore/monitor-private.h"
+#include "MagickCore/pixel-accessor.h"
+#include "MagickCore/pixel-private.h"
+#include "MagickCore/quantize.h"
+#include "MagickCore/quantum-private.h"
+#include "MagickCore/resize.h"
+#include "MagickCore/resource_.h"
+#include "MagickCore/splay-tree.h"
+#include "MagickCore/static.h"
+#include "MagickCore/string_.h"
+#include "MagickCore/thread-private.h"
+#include "MagickCore/module.h"
+#include "MagickCore/threshold.h"
+#include "MagickCore/utility.h"
 
 /*
   Definitions
@@ -83,8 +83,8 @@
 /*
   Macros
 */
-#define SIXEL_RGB(r, g, b) (((r) << 16) + ((g) << 8) +  (b))
-#define SIXEL_PALVAL(n,a,m) (((n) * (a) + ((m) / 2)) / (m))
+#define SIXEL_RGB(r, g, b) ((int) (((ssize_t) (r) << 16) + ((g) << 8) +  (b)))
+#define SIXEL_PALVAL(n,a,m) ((int) (((ssize_t) (n) * (a) + ((m) / 2)) / (m)))
 #define SIXEL_XRGB(r,g,b) SIXEL_RGB(SIXEL_PALVAL(r, 255, 100), SIXEL_PALVAL(g, 255, 100), SIXEL_PALVAL(b, 255, 100))
 
 /*
@@ -142,7 +142,7 @@ static int const sixel_default_color_table[] = {
   Forward declarations.
 */
 static MagickBooleanType
-  WriteSIXELImage(const ImageInfo *,Image *);
+  WriteSIXELImage(const ImageInfo *,Image *,ExceptionInfo *);
 
 static int hue_to_rgb(int n1, int n2, int hue)
 {
@@ -179,9 +179,9 @@ static int hls_to_rgb(int hue, int lum, int sat)
         R = G = B = (lum * RGBMAX) / HLSMAX;
     } else {
         if (lum <= (HLSMAX / 2)) {
-            Magic2 = (lum * (HLSMAX + sat) + (HLSMAX / 2)) / HLSMAX;
+            Magic2 = (int) (((ssize_t) lum * (HLSMAX + sat) + (HLSMAX / 2)) / HLSMAX);
         } else {
-            Magic2 = lum + sat - ((lum * sat) + (HLSMAX / 2)) / HLSMAX;
+            Magic2 = (int) (lum + sat - (((ssize_t) lum * sat) + (HLSMAX / 2)) / HLSMAX);
         }
         Magic1 = 2 * lum - Magic2;
 
@@ -201,9 +201,10 @@ static unsigned char *get_params(unsigned char *p, int *param, int *len)
         while (*p == ' ' || *p == '\t') {
             p++;
         }
-        if (isdigit(*p)) {
-            for (n = 0; isdigit(*p); p++) {
-                n = n * 10 + (*p - '0');
+        if (isdigit((int) ((unsigned char) *p))) {
+            for (n = 0; isdigit((int) ((unsigned char) *p)); p++) {
+                if (n <= (INT_MAX/10))
+                  n = (int) ((ssize_t) n * 10 + (*p - '0'));
             }
             if (*len < 10) {
                 param[(*len)++] = n;
@@ -226,12 +227,14 @@ static unsigned char *get_params(unsigned char *p, int *param, int *len)
 }
 
 /* convert sixel data into indexed pixel bytes and palette data */
-MagickBooleanType sixel_decode(unsigned char              /* in */  *p,         /* sixel bytes */
+MagickBooleanType sixel_decode(Image *image,
+                               unsigned char              /* in */  *p,         /* sixel bytes */
                                unsigned char              /* out */ **pixels,   /* decoded pixels */
                                size_t                     /* out */ *pwidth,    /* image width */
                                size_t                     /* out */ *pheight,   /* image height */
                                unsigned char              /* out */ **palette,  /* ARGB palette */
-                               size_t                     /* out */ *ncolors    /* palette size (<= 256) */)
+                               size_t                     /* out */ *ncolors,    /* palette size (<= 256) */
+  ExceptionInfo *exception)
 {
     int n, i, r, g, b, sixel_vertical_mask, c;
     int posision_x, posision_y;
@@ -239,13 +242,15 @@ MagickBooleanType sixel_decode(unsigned char              /* in */  *p,
     int attributed_pan, attributed_pad;
     int attributed_ph, attributed_pv;
     int repeat_count, color_index, max_color_index = 2, background_color_index;
-    int param[10];
+    int param[10] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
     int sixel_palet[SIXEL_PALETTE_MAX];
     unsigned char *imbuf, *dmbuf;
     int imsx, imsy;
     int dmsx, dmsy;
     int y;
+    size_t extent,offset;
 
+    extent=strlen((char *) p);
     posision_x = posision_y = 0;
     max_x = max_y = 0;
     attributed_pan = 2;
@@ -257,7 +262,9 @@ MagickBooleanType sixel_decode(unsigned char              /* in */  *p,
 
     imsx = 2048;
     imsy = 2048;
-    imbuf = (unsigned char *) AcquireQuantumMemory(imsx * imsy,1);
+    if (SetImageExtent(image,imsx,imsy,exception) == MagickFalse)
+      return(MagickFalse);
+    imbuf = (unsigned char *) AcquireQuantumMemory(imsx , imsy);
 
     if (imbuf == NULL) {
         return(MagickFalse);
@@ -284,7 +291,7 @@ MagickBooleanType sixel_decode(unsigned char              /* in */  *p,
         sixel_palet[n] = SIXEL_RGB(255, 255, 255);
     }
 
-    (void) ResetMagickMemory(imbuf, background_color_index, imsx * imsy);
+    (void) memset(imbuf, background_color_index, (size_t) imsx * imsy);
 
     while (*p != '\0') {
         if ((p[0] == '\033' && p[1] == 'P') || *p == 0x90) {
@@ -334,8 +341,8 @@ MagickBooleanType sixel_decode(unsigned char              /* in */  *p,
                     if (param[2] == 0) {
                         param[2] = 10;
                     }
-                    attributed_pan = attributed_pan * param[2] / 10;
-                    attributed_pad = attributed_pad * param[2] / 10;
+                    attributed_pan = (int) (((ssize_t) attributed_pan * param[2]) / 10);
+                    attributed_pad = (int) (((ssize_t) attributed_pad * param[2]) / 10);
                     if (attributed_pan <= 0) attributed_pan = 1;
                     if (attributed_pad <= 0) attributed_pad = 1;
                 }
@@ -349,8 +356,8 @@ MagickBooleanType sixel_decode(unsigned char              /* in */  *p,
 
             if (n > 0) attributed_pad = param[0];
             if (n > 1) attributed_pan = param[1];
-            if (n > 2 && param[2] > 0) attributed_ph = param[2];
-            if (n > 3 && param[3] > 0) attributed_pv = param[3];
+            if (n > 2 && param[2] > 0) attributed_ph = param[2] & 0xffff;
+            if (n > 3 && param[3] > 0) attributed_pv = param[3] & 0xffff;
 
             if (attributed_pan <= 0) attributed_pan = 1;
             if (attributed_pad <= 0) attributed_pad = 1;
@@ -358,14 +365,16 @@ MagickBooleanType sixel_decode(unsigned char              /* in */  *p,
             if (imsx < attributed_ph || imsy < attributed_pv) {
                 dmsx = imsx > attributed_ph ? imsx : attributed_ph;
                 dmsy = imsy > attributed_pv ? imsy : attributed_pv;
-                dmbuf = (unsigned char *) AcquireQuantumMemory(dmsx * dmsy,1);
+                if (SetImageExtent(image,dmsx,dmsy,exception) == MagickFalse)
+                  break;
+                dmbuf = (unsigned char *) AcquireQuantumMemory(dmsx , dmsy);
                 if (dmbuf == (unsigned char *) NULL) {
                     imbuf = (unsigned char *) RelinquishMagickMemory(imbuf);
                     return (MagickFalse);
                 }
-                (void) ResetMagickMemory(dmbuf, background_color_index, dmsx * dmsy);
+                (void) memset(dmbuf, background_color_index, (size_t) dmsx * dmsy);
                 for (y = 0; y < imsy; ++y) {
-                    (void) CopyMagickMemory(dmbuf + dmsx * y, imbuf + imsx * y, imsx);
+                    (void) memcpy(dmbuf + dmsx * y, imbuf + imsx * y, imsx);
                 }
                 imbuf = (unsigned char *) RelinquishMagickMemory(imbuf);
                 imsx = dmsx;
@@ -377,8 +386,10 @@ MagickBooleanType sixel_decode(unsigned char              /* in */  *p,
             /* DECGRI Graphics Repeat Introducer ! Pn Ch */
             p = get_params(++p, param, &n);
 
-            if (n > 0) {
+            if ((n > 0) && (param[0] > 0)) {
                 repeat_count = param[0];
+                if (repeat_count > (ssize_t) extent)
+                  break;
             }
 
         } else if (*p == '#') {
@@ -398,7 +409,7 @@ MagickBooleanType sixel_decode(unsigned char              /* in */  *p,
                     if (param[2] > 360) param[2] = 360;
                     if (param[3] > 100) param[3] = 100;
                     if (param[4] > 100) param[4] = 100;
-                    sixel_palet[color_index] = hls_to_rgb(param[2] * 100 / 360, param[3], param[4]);
+                    sixel_palet[color_index] = hls_to_rgb((int) ((ssize_t) param[2] * 100 / 360), param[3], param[4]);
                 } else if (param[1] == 2) {    /* RGB */
                     if (param[2] > 100) param[2] = 100;
                     if (param[3] > 100) param[3] = 100;
@@ -432,14 +443,16 @@ MagickBooleanType sixel_decode(unsigned char              /* in */  *p,
 
                 dmsx = nx;
                 dmsy = ny;
-                dmbuf = (unsigned char *) AcquireQuantumMemory(dmsx * dmsy,1);
+                if (SetImageExtent(image,dmsx,dmsy,exception) == MagickFalse)
+                  break;
+                dmbuf = (unsigned char *) AcquireQuantumMemory(dmsx , dmsy);
                 if (dmbuf == (unsigned char *) NULL) {
                     imbuf = (unsigned char *) RelinquishMagickMemory(imbuf);
                     return (MagickFalse);
                 }
-                (void) ResetMagickMemory(dmbuf, background_color_index, dmsx * dmsy);
+                (void) memset(dmbuf, background_color_index, (size_t) dmsx * dmsy);
                 for (y = 0; y < imsy; ++y) {
-                    (void) CopyMagickMemory(dmbuf + dmsx * y, imbuf + imsx * y, imsx);
+                    (void) memcpy(dmbuf + dmsx * y, imbuf + imsx * y, imsx);
                 }
                 imbuf = (unsigned char *) RelinquishMagickMemory(imbuf);
                 imsx = dmsx;
@@ -459,7 +472,13 @@ MagickBooleanType sixel_decode(unsigned char              /* in */  *p,
                 if (repeat_count <= 1) {
                     for (i = 0; i < 6; i++) {
                         if ((b & sixel_vertical_mask) != 0) {
-                            imbuf[imsx * (posision_y + i) + posision_x] = color_index;
+                            offset=(size_t) imsx * (posision_y + i) + posision_x;
+                            if (offset >= (size_t) imsx * imsy)
+                              {
+                                imbuf = (unsigned char *) RelinquishMagickMemory(imbuf);
+                                return (MagickFalse);
+                              }
+                            imbuf[offset] = color_index;
                             if (max_x < posision_x) {
                                 max_x = posision_x;
                             }
@@ -482,7 +501,13 @@ MagickBooleanType sixel_decode(unsigned char              /* in */  *p,
                                 c <<= 1;
                             }
                             for (y = posision_y + i; y < posision_y + i + n; ++y) {
-                                (void) ResetMagickMemory(imbuf + imsx * y + posision_x, color_index, repeat_count);
+                                offset=(size_t) imsx * y + posision_x;
+                                if (offset + repeat_count >= (size_t) imsx * imsy)
+                                  {
+                                    imbuf = (unsigned char *) RelinquishMagickMemory(imbuf);
+                                    return (MagickFalse);
+                                  }
+                                (void) memset(imbuf + offset, color_index, repeat_count);
                             }
                             if (max_x < (posision_x + repeat_count - 1)) {
                                 max_x = posision_x + repeat_count - 1;
@@ -515,12 +540,17 @@ MagickBooleanType sixel_decode(unsigned char              /* in */  *p,
     if (imsx > max_x || imsy > max_y) {
         dmsx = max_x;
         dmsy = max_y;
-        if ((dmbuf = (unsigned char *) AcquireQuantumMemory(dmsx * dmsy,1)) == NULL) {
+        if (SetImageExtent(image,dmsx,dmsy,exception) == MagickFalse)
+          {
+            imbuf = (unsigned char *) RelinquishMagickMemory(imbuf);
+            return (MagickFalse);
+          }
+        if ((dmbuf = (unsigned char *) AcquireQuantumMemory(dmsx , dmsy)) == NULL) {
             imbuf = (unsigned char *) RelinquishMagickMemory(imbuf);
             return (MagickFalse);
         }
         for (y = 0; y < dmsy; ++y) {
-            (void) CopyMagickMemory(dmbuf + dmsx * y, imbuf + imsx * y, dmsx);
+            (void) memcpy(dmbuf + dmsx * y, imbuf + imsx * y, dmsx);
         }
         imbuf = (unsigned char *) RelinquishMagickMemory(imbuf);
         imsx = dmsx;
@@ -533,6 +563,8 @@ MagickBooleanType sixel_decode(unsigned char              /* in */  *p,
     *pheight = imsy;
     *ncolors = max_color_index + 1;
     *palette = (unsigned char *) AcquireQuantumMemory(*ncolors,4);
+    if (*palette == (unsigned char *) NULL)
+      return(MagickFalse);
     for (n = 0; n < (ssize_t) *ncolors; ++n) {
         (*palette)[n * 4 + 0] = sixel_palet[n] >> 16 & 0xff;
         (*palette)[n * 4 + 1] = sixel_palet[n] >> 8 & 0xff;
@@ -547,6 +579,8 @@ sixel_output_t *sixel_output_create(Image *image)
     sixel_output_t *output;
 
     output = (sixel_output_t *) AcquireQuantumMemory(sizeof(sixel_output_t) + SIXEL_OUTPUT_PACKET_SIZE * 2, 1);
+    if (output == (sixel_output_t *) NULL)
+      return((sixel_output_t *) NULL);
     output->has_8bit_control = 0;
     output->save_pixel = 0;
     output->save_count = 0;
@@ -563,7 +597,7 @@ static void sixel_advance(sixel_output_t *context, int nwrite)
 {
     if ((context->pos += nwrite) >= SIXEL_OUTPUT_PACKET_SIZE) {
         WriteBlob(context->image,SIXEL_OUTPUT_PACKET_SIZE,context->buffer);
-        CopyMagickMemory(context->buffer,
+        memmove(context->buffer,
                context->buffer + SIXEL_OUTPUT_PACKET_SIZE,
                (context->pos -= SIXEL_OUTPUT_PACKET_SIZE));
     }
@@ -686,10 +720,10 @@ static MagickBooleanType sixel_encode_impl(unsigned char *pixels, size_t width,s
     int x, y, i, n, c;
     int left, right;
     int pix;
-    size_t len;
     unsigned char *map;
     sixel_node_t *np, *tp, top;
     int nwrite;
+    size_t len;
 
     context->pos = 0;
 
@@ -702,7 +736,7 @@ static MagickBooleanType sixel_encode_impl(unsigned char *pixels, size_t width,s
     if ((map = (unsigned char *)AcquireQuantumMemory(len, sizeof(unsigned char))) == NULL) {
         return (MagickFalse);
     }
-    (void) ResetMagickMemory(map, 0, len);
+    (void) memset(map, 0, len);
 
     if (context->has_8bit_control) {
         nwrite = sprintf((char *)context->buffer, "\x90" "0;0;0" "q");
@@ -817,7 +851,7 @@ static MagickBooleanType sixel_encode_impl(unsigned char *pixels, size_t width,s
                 x = 0;
             }
 
-            x = sixel_put_node(context, x, np, ncolors, keycolor);
+            x = sixel_put_node(context, x, np, (int) ncolors, keycolor);
             sixel_node_del(context, np);
             np = context->node_top;
 
@@ -827,7 +861,7 @@ static MagickBooleanType sixel_encode_impl(unsigned char *pixels, size_t width,s
                     continue;
                 }
 
-                x = sixel_put_node(context, x, np, ncolors, keycolor);
+                x = sixel_put_node(context, x, np, (int) ncolors, keycolor);
                 sixel_node_del(context, np);
                 np = context->node_top;
             }
@@ -842,7 +876,7 @@ static MagickBooleanType sixel_encode_impl(unsigned char *pixels, size_t width,s
         }
 
         i = 0;
-        (void) ResetMagickMemory(map, 0, len);
+        (void) memset(map, 0, len);
     }
 
     if (context->has_8bit_control) {
@@ -860,7 +894,7 @@ static MagickBooleanType sixel_encode_impl(unsigned char *pixels, size_t width,s
 
     /* flush buffer */
     if (context->pos > 0) {
-        (void) WriteBlob(context->image,context->pos,context->buffer);
+        WriteBlob(context->image,context->pos,context->buffer);
     }
 
     RelinquishNodesAndMap;
@@ -884,7 +918,8 @@ static MagickBooleanType sixel_encode_impl(unsigned char *pixels, size_t width,s
 %
 %  The format of the IsSIXEL method is:
 %
-%      MagickBooleanType IsSIXEL(const unsigned char *magick,const size_t length)
+%      MagickBooleanType IsSIXEL(const unsigned char *magick,
+%        const size_t length)
 %
 %  A description of each parameter follows:
 %
@@ -894,7 +929,8 @@ static MagickBooleanType sixel_encode_impl(unsigned char *pixels, size_t width,s
 %    o length: Specifies the length of the magick string.
 %
 */
-static MagickBooleanType IsSIXEL(const unsigned char *magick,const size_t length)
+static MagickBooleanType IsSIXEL(const unsigned char *magick,
+  const size_t length)
 {
   const unsigned char
     *end = magick + length;
@@ -930,7 +966,8 @@ static MagickBooleanType IsSIXEL(const unsigned char *magick,const size_t length
 %
 %  The format of the ReadSIXELImage method is:
 %
-%      Image *ReadSIXELImage(const ImageInfo *image_info,ExceptionInfo *exception)
+%      Image *ReadSIXELImage(const ImageInfo *image_info,
+%        ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -953,14 +990,11 @@ static Image *ReadSIXELImage(const ImageInfo *image_info,ExceptionInfo *exceptio
   register char
     *p;
 
-  register IndexPacket
-    *indexes;
-
   register ssize_t
     x;
 
-  register PixelPacket
-    *r;
+  register Quantum
+    *q;
 
   size_t
     length;
@@ -978,13 +1012,13 @@ static Image *ReadSIXELImage(const ImageInfo *image_info,ExceptionInfo *exceptio
     Open image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
-  image=AcquireImage(image_info);
+  assert(exception->signature == MagickCoreSignature);
+  image=AcquireImage(image_info,exception);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
@@ -994,8 +1028,9 @@ static Image *ReadSIXELImage(const ImageInfo *image_info,ExceptionInfo *exceptio
   /*
     Read SIXEL file.
   */
-  length=MaxTextExtent;
-  sixel_buffer=(char *) AcquireQuantumMemory((size_t) length,sizeof(*sixel_buffer));
+  length=MagickPathExtent;
+  sixel_buffer=(char *) AcquireQuantumMemory((size_t) length+MagickPathExtent,
+    sizeof(*sixel_buffer));
   p=sixel_buffer;
   if (sixel_buffer != (char *) NULL)
     while (ReadBlobString(image,p) != (char *) NULL)
@@ -1005,36 +1040,41 @@ static Image *ReadSIXELImage(const ImageInfo *image_info,ExceptionInfo *exceptio
       if ((*p == '}') && (*(p+1) == ';'))
         break;
       p+=strlen(p);
-      if ((size_t) (p-sixel_buffer+MaxTextExtent) < length)
+      if ((size_t) (p-sixel_buffer+MagickPathExtent+1) < length)
         continue;
       length<<=1;
-      sixel_buffer=(char *) ResizeQuantumMemory(sixel_buffer,length+MaxTextExtent,
-        sizeof(*sixel_buffer));
+      sixel_buffer=(char *) ResizeQuantumMemory(sixel_buffer,length+
+        MagickPathExtent+1,sizeof(*sixel_buffer));
       if (sixel_buffer == (char *) NULL)
         break;
       p=sixel_buffer+strlen(sixel_buffer);
     }
   if (sixel_buffer == (char *) NULL)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-
+  sixel_buffer[length]='\0';
   /*
     Decode SIXEL
   */
-  if (sixel_decode((unsigned char *)sixel_buffer, &sixel_pixels, &image->columns, &image->rows, &sixel_palette, &image->colors) == MagickFalse)
+  sixel_pixels=(unsigned char *) NULL;
+  if (sixel_decode(image,(unsigned char *) sixel_buffer,&sixel_pixels,&image->columns,&image->rows,&sixel_palette,&image->colors,exception) == MagickFalse)
     {
       sixel_buffer=(char *) RelinquishMagickMemory(sixel_buffer);
+      if (sixel_pixels != (unsigned char *) NULL)
+        sixel_pixels=(unsigned char *) RelinquishMagickMemory(sixel_pixels);
       ThrowReaderException(CorruptImageError,"CorruptImage");
     }
   sixel_buffer=(char *) RelinquishMagickMemory(sixel_buffer);
   image->depth=24;
   image->storage_class=PseudoClass;
-  status=SetImageExtent(image,image->columns,image->rows);
+  status=SetImageExtent(image,image->columns,image->rows,exception);
   if (status == MagickFalse)
     {
-      InheritException(exception,&image->exception);
+      sixel_pixels=(unsigned char *) RelinquishMagickMemory(sixel_pixels);
+      sixel_palette=(unsigned char *) RelinquishMagickMemory(sixel_palette);
       return(DestroyImageList(image));
     }
-  if (AcquireImageColormap(image,image->colors) == MagickFalse)
+
+  if (AcquireImageColormap(image,image->colors, exception) == MagickFalse)
     {
       sixel_pixels=(unsigned char *) RelinquishMagickMemory(sixel_pixels);
       sixel_palette=(unsigned char *) RelinquishMagickMemory(sixel_palette);
@@ -1054,15 +1094,14 @@ static Image *ReadSIXELImage(const ImageInfo *image_info,ExceptionInfo *exceptio
       */
       for (y=0; y < (ssize_t) image->rows; y++)
       {
-        r=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
-        if (r == (PixelPacket *) NULL)
+        q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
+        if (q == (Quantum *) NULL)
           break;
-        indexes=GetAuthenticIndexQueue(image);
         for (x=0; x < (ssize_t) image->columns; x++)
         {
           j=(ssize_t) sixel_pixels[y * image->columns + x];
-          SetPixelIndex(indexes+x,j);
-          r++;
+          SetPixelIndex(image,j,q);
+          q+=GetPixelChannels(image);
         }
         if (SyncAuthenticPixels(image,exception) == MagickFalse)
           break;
@@ -1111,21 +1150,17 @@ ModuleExport size_t RegisterSIXELImage(void)
   MagickInfo
     *entry;
 
-  entry=SetMagickInfo("SIXEL");
+  entry=AcquireMagickInfo("SIXEL","SIXEL","DEC SIXEL Graphics Format");
   entry->decoder=(DecodeImageHandler *) ReadSIXELImage;
   entry->encoder=(EncodeImageHandler *) WriteSIXELImage;
   entry->magick=(IsImageFormatHandler *) IsSIXEL;
-  entry->adjoin=MagickFalse;
-  entry->description=ConstantString("DEC SIXEL Graphics Format");
-  entry->module=ConstantString("SIXEL");
+  entry->flags^=CoderAdjoinFlag;
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("SIX");
+  entry=AcquireMagickInfo("SIXEL","SIX","DEC SIXEL Graphics Format");
   entry->decoder=(DecodeImageHandler *) ReadSIXELImage;
   entry->encoder=(EncodeImageHandler *) WriteSIXELImage;
   entry->magick=(IsImageFormatHandler *) IsSIXEL;
-  entry->adjoin=MagickFalse;
-  entry->description=ConstantString("DEC SIXEL Graphics Format");
-  entry->module=ConstantString("SIX");
+  entry->flags^=CoderAdjoinFlag;
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
@@ -1182,16 +1217,14 @@ ModuleExport void UnregisterSIXELImage(void)
 %    o exception: return any errors or warnings in this structure.
 %
 */
-static MagickBooleanType WriteSIXELImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteSIXELImage(const ImageInfo *image_info,
+  Image *image,ExceptionInfo *exception)
 {
-  ExceptionInfo
-    *exception;
-
   MagickBooleanType
     status;
 
-  register const IndexPacket
-    *indexes;
+  register const Quantum
+    *q;
 
   register ssize_t
     i,
@@ -1205,29 +1238,28 @@ static MagickBooleanType WriteSIXELImage(const ImageInfo *image_info,Image *imag
     *output;
 
   unsigned char
-    sixel_palette[256 * 3],
+    sixel_palette[256*3],
     *sixel_pixels;
 
   /*
     Open output image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  exception=(&image->exception);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,exception);
   if (status == MagickFalse)
     return(status);
   if (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse)
-    (void) TransformImageColorspace(image,sRGBColorspace);
+    (void) TransformImageColorspace(image,sRGBColorspace,exception);
   opacity=(-1);
-  if (image->matte == MagickFalse)
+  if (image->alpha_trait == UndefinedPixelTrait)
     {
       if ((image->storage_class == DirectClass) || (image->colors > 256))
-        (void) SetImageType(image,PaletteType);
+        (void) SetImageType(image,PaletteType,exception);
     }
   else
     {
@@ -1239,34 +1271,34 @@ static MagickBooleanType WriteSIXELImage(const ImageInfo *image_info,Image *imag
         Identify transparent colormap index.
       */
       if ((image->storage_class == DirectClass) || (image->colors > 256))
-        (void) SetImageType(image,PaletteBilevelMatteType);
+        (void) SetImageType(image,PaletteBilevelAlphaType,exception);
       for (i=0; i < (ssize_t) image->colors; i++)
-        if (image->colormap[i].opacity != OpaqueOpacity)
+        if (image->colormap[i].alpha != OpaqueAlpha)
           {
             if (opacity < 0)
               {
                 opacity=i;
                 continue;
               }
-            alpha=(MagickRealType) image->colormap[i].opacity;
-            beta=(MagickRealType) image->colormap[opacity].opacity;
-            if (alpha > beta)
+            alpha=image->colormap[i].alpha;
+            beta=image->colormap[opacity].alpha;
+            if (alpha < beta)
               opacity=i;
           }
       if (opacity == -1)
         {
-          (void) SetImageType(image,PaletteBilevelMatteType);
+          (void) SetImageType(image,PaletteBilevelAlphaType,exception);
           for (i=0; i < (ssize_t) image->colors; i++)
-            if (image->colormap[i].opacity != OpaqueOpacity)
+            if (image->colormap[i].alpha != OpaqueAlpha)
               {
                 if (opacity < 0)
                   {
                     opacity=i;
                     continue;
                   }
-                alpha=(MagickRealType) image->colormap[i].opacity;
-                beta=(MagickRealType) image->colormap[opacity].opacity;
-                if (alpha > beta)
+                alpha=image->colormap[i].alpha;
+                beta=image->colormap[opacity].alpha;
+                if (alpha < beta)
                   opacity=i;
               }
         }
@@ -1282,28 +1314,39 @@ static MagickBooleanType WriteSIXELImage(const ImageInfo *image_info,Image *imag
   */
   for (i=0; i < (ssize_t) image->colors; i++)
   {
-    sixel_palette[i * 3 + 0] = ScaleQuantumToChar(image->colormap[i].red);
-    sixel_palette[i * 3 + 1] = ScaleQuantumToChar(image->colormap[i].green);
-    sixel_palette[i * 3 + 2] = ScaleQuantumToChar(image->colormap[i].blue);
+    sixel_palette[3*i+0]=ScaleQuantumToChar(image->colormap[i].red);
+    sixel_palette[3*i+1]=ScaleQuantumToChar(image->colormap[i].green);
+    sixel_palette[3*i+2]=ScaleQuantumToChar(image->colormap[i].blue);
   }
 
   /*
     Define SIXEL pixels.
   */
   output = sixel_output_create(image);
-  sixel_pixels =(unsigned char *) AcquireQuantumMemory(image->columns * image->rows,1);
+  if (output == (sixel_output_t *) NULL)
+    ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
+  sixel_pixels=(unsigned char *) AcquireQuantumMemory(image->columns,
+    image->rows*sizeof(*sixel_pixels));
+  if (sixel_pixels == (unsigned char *) NULL)
+    {
+      output = (sixel_output_t *) RelinquishMagickMemory(output);
+      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
+    }
   for (y=0; y < (ssize_t) image->rows; y++)
   {
-    (void) GetVirtualPixels(image,0,y,image->columns,1,exception);
-    indexes=GetVirtualIndexQueue(image);
+    q=GetVirtualPixels(image,0,y,image->columns,1,exception);
+    if (q == (Quantum *) NULL)
+      break;
     for (x=0; x < (ssize_t) image->columns; x++)
-      sixel_pixels[y * image->columns + x] = (unsigned char) ((ssize_t) GetPixelIndex(indexes + x));
+    {
+      sixel_pixels[y*image->columns+x]= ((ssize_t) GetPixelIndex(image,q));
+      q+=GetPixelChannels(image);
+    }
   }
-  status = sixel_encode_impl(sixel_pixels, image->columns, image->rows,
-                          sixel_palette, image->colors, -1,
-                          output);
-  sixel_pixels =(unsigned char *) RelinquishMagickMemory(sixel_pixels);
-  output = (sixel_output_t *) RelinquishMagickMemory(output);
+  status = sixel_encode_impl(sixel_pixels,image->columns,image->rows,
+    sixel_palette,image->colors,-1,output);
+  sixel_pixels=(unsigned char *) RelinquishMagickMemory(sixel_pixels);
+  output=(sixel_output_t *) RelinquishMagickMemory(output);
   (void) CloseBlob(image);
   return(status);
 }
